// ***********************************************************************************************************************
// ** File:         enc28j60.c                                                                                          **
// ** Title:        Driver for Microchip ENC28J60 Ethernet Driver                                                       **
// ** CPU:          ATMega644                                                                                           **
// ** Created:      28.02.2008                                                                                          **
// ** Revised:      28.02.2008                                                                                          **
// ** Version:      1.0                                                                                                 **
// ** Note:                                                                                                             **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// **                                                                                                                   **
// ***********************************************************************************************************************

// INCLUDES **************************************************************************************************************
#include "global.h"                                                         //
// ***********************************************************************************************************************

// DEFINES ***************************************************************************************************************
                                                                            //
// ***********************************************************************************************************************

// VARIABLES *************************************************************************************************************
unsigned char   Enc28j60Bank;                                               // Aktuelle Bank vom ENC wird hier zwischengespeichert
unsigned int    NextPacketPtr;                                              // Zeiger auf das nchste paket im ENC Buffer
unsigned char   buffer [NICBUFFERSIZE];                                     // ENC Sende und Empfangs Buffer
unsigned char   MyMac [6];                                                  // Buffer fr die eigene MAC Adresse
//volatile unsigned char   ENC28J60_INT_Flag;                                          //
// ***********************************************************************************************************************

// ENC28J60 Lesen mit Angabe der Bank ************************************************************************************
unsigned char enc28j60ReadOp(unsigned char op, unsigned char address)       //
{                                                                           //
    unsigned char dat;                                                      // dat ist ein Byte gross
    ENC28J60_CS = 0;                                                        // ENC_CS auf Low
    spi_out (op | (address & ADDR_MASK));                                   // adrout an ENC_Dout ausgeben 
    dat = spi_in();                                                         // Daten Einlesen
    if(address & 0x80)                                                      // Wenn Phy Register gelesen werden, dann muss das Dummy Byte verworfen werden
    {                                                                       //
        dat = spi_in();                                                     // Nochmals Einlesen um an die Daten zu kommen
    }                                                                       //
    ENC28J60_CS = 1;                                                        // ENC_CS auf High
    return dat;                                                             // dat an die aufrufende Funktion weitergeben
}                                                                           //
// ***********************************************************************************************************************

// ENC28J60 schreiben mit Angabe der Bank ********************************************************************************
void enc28j60WriteOp(unsigned char op, unsigned char address, unsigned char dat)
{                                                                           //
    ENC28J60_CS = 0;                                                        // ENC_CS auf Low
    spi_out (op | (address & ADDR_MASK));                                   // Adresse an ENC Dout ausgeben
    spi_out (dat);                                                          // dat an ENC_Dout ausgeben 
    ENC28J60_CS = 1;                                                        // ENC_CS auf High
}                                                                           //
// ***********************************************************************************************************************

// ENC28J60 Buffer Lesen *************************************************************************************************
void enc28j60ReadBuffer(unsigned int len, unsigned char* dat)               //
{                                                                           //
    ENC28J60_CS = 0;                                                        // ENC_CS auf Low
    spi_out (ENC28J60_READ_BUF_MEM);                                        // RBM commando ausgeben
    while(len--)                                                            // solange Daten einlesen wie in len angegeben
    {                                                                       //
        *dat++ = spi_in();                                                  // spiIn in *dat schieben
    }                                                                       //
    ENC28J60_CS = 1;                                                        // release CS
}                                                                           //
// ***********************************************************************************************************************

// ENC28J60 Buffer schreiben *********************************************************************************************
void enc28j60WriteBuffer(unsigned int len, unsigned char* dat)              //
{                                                                           //
    ENC28J60_CS = 0;                                                        // ENC_CS auf Low
    spi_out (ENC28J60_WRITE_BUF_MEM);                                       // WBM comanndo ausgeben
    while(len--)                                                            // solange Daten schreiben wie in len angegeben
    {                                                                       //
        spi_out (*dat++);                                                   // Daten in den Buffer schieben
    }                                                                       //
    ENC28J60_CS = 1;                                                        // release CS
}                                                                           //
// ***********************************************************************************************************************

// ENC28J60 Bank umschalten **********************************************************************************************
void enc28j60SetBank(unsigned char address)                                 //
{                                                                           //
    if((address & BANK_MASK) != Enc28j60Bank)                               // set the bank (if needed)
    {                                                                       //
        enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR,ECON1,(ECON1_BSEL1|ECON1_BSEL0));// BSEL 0 + 1 lschen
        enc28j60WriteOp(ENC28J60_BIT_FIELD_SET,ECON1,(address & BANK_MASK)>>5);// BSEL 0 + 1 auf die ensprechende Bank setzen
        Enc28j60Bank = (address & BANK_MASK);                               // Globale Variable auf die gesetzte Bank setzen
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

// Control Register lesen ************************************************************************************************
unsigned char enc28j60Read(unsigned char address)                           //
{                                                                           //
    enc28j60SetBank(address);                                               // Bank umschalten
    return enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address);                 // Control Register lesen
}                                                                           //
// ***********************************************************************************************************************

// Control Register schreiben ********************************************************************************************
void enc28j60Write(unsigned char address, unsigned char dat)                //
{                                                                           //
    enc28j60SetBank(address);                                               // Bank umschalten
    enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, dat);                 // Control Register schreiben
}                                                                           //
// ***********************************************************************************************************************

// Phy Register schreiben ************************************************************************************************
void enc28j60PhyWrite(unsigned char address, unsigned int dat)              //
{                                                                           //
    enc28j60Write(MIREGADR, address);                                       // set the PHY register address
    enc28j60Write(MIWRL, dat);                                              // write the PHY data
    enc28j60Write(MIWRH, dat>>8);                                           //
    while(enc28j60Read(MISTAT) & MISTAT_BUSY);                              // wait until the PHY write completes
}                                                                           //
// ***********************************************************************************************************************

// Phy Register lesen ****************************************************************************************************
unsigned int enc28j60PhyRead(unsigned char address)                         //
{                                                                           //
    unsigned int dat = 0;                                                   //
    enc28j60Write(MIREGADR, address);                                       // set the PHY register address
    enc28j60Write(MICMD, MICMD_MIIRD);                                      // set the MICMD.MIIRD bit
    while(enc28j60Read(MISTAT) & MISTAT_BUSY);                              // wait until the PHY read completes
    enc28j60Write(MICMD, 0x00);                                             // clear the MICMD.MIIRD bit
    dat  = enc28j60Read(MIRDL);                                             //
    dat |= enc28j60Read(MIRDH)<<8;                                          //
    return dat;                                                             // return the data
}                                                                           //
// ***********************************************************************************************************************

// ENC Initialisieren ****************************************************************************************************
void enc28j60Init(void)                                                     //
{                                                                           //
    enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);           // perform system reset
    _delay_ms (20);                                                         // ca 20 mSec warten nach reset 

    while(!(enc28j60Read(ESTAT) & ESTAT_CLKRDY));                           // check CLKRDY bit to see if reset is complete
    
    // do bank 0 stuff
    NextPacketPtr = RXSTART_INIT;                                           // initialize receive buffer
    enc28j60Write(ERXSTL, RXSTART_INIT&0xFF);                               // 16-bit transfers, must write low byte first
    enc28j60Write(ERXSTH, RXSTART_INIT>>8);                                 // set receive buffer start address
    enc28j60Write(ERXRDPTL, RXSTART_INIT&0xFF);                             // set receive pointer address
    enc28j60Write(ERXRDPTH, RXSTART_INIT>>8);                               //
    enc28j60Write(ERXNDL, RXSTOP_INIT&0xFF);                                // ERXND defaults to 0x1FFF (end of ram)
    enc28j60Write(ERXNDH, RXSTOP_INIT>>8);                                  // set receive buffer end
    enc28j60Write(ETXSTL, TXSTART_INIT&0xFF);                               // set transmit buffer start
    enc28j60Write(ETXSTH, TXSTART_INIT>>8);                                 // ETXST defaults to 0x0000 (beginnging of ram)
    
    // do bank 2 stuff    
    enc28j60Write(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);       // enable MAC receive
    enc28j60Write(MACON2, 0x00);                                            // bring MAC out of reset
    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET                                  //
        , MACON3, MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);            // enable automatic padding and CRC operations
    enc28j60Write(MAIPGL, 0x12);                                            // set inter-frame gap (non-back-to-back)
    enc28j60Write(MAIPGH, 0x0C);                                            //
    enc28j60Write(MABBIPG, 0x12);                                           // set inter-frame gap (back-to-back)
    enc28j60Write(MAMXFLL, MAX_FRAMELEN&0xFF);                              // Set the maximum packet size which the controller will accept
    enc28j60Write(MAMXFLH, MAX_FRAMELEN>>8);                                //
    
    //do bank 3 stuff
    enc28j60Write(MAADR1, MyMac [0]);                                       // write MAC address
    enc28j60Write(MAADR2, MyMac [1]);                                       // NOTE: MAC address in ENC28J60 is byte-backward
    enc28j60Write(MAADR3, MyMac [2]);                                       //
    enc28j60Write(MAADR4, MyMac [3]);                                       //
    enc28j60Write(MAADR5, MyMac [4]);                                       //
    enc28j60Write(MAADR6, MyMac [5]);                                       //
    enc28j60Write(ECOCON, 0x00);                                            // ENC Clock out disabled
 
    enc28j60Write(ERXFCON, ERXFCON_BCEN|ERXFCON_MCEN|ERXFCON_UCEN);         // receive broadcast, multicast and unicast packets
 
    //do Phy Register stuff
    enc28j60PhyWrite(PHCON2, PHCON2_HDLDIS);                                // no loopback of transmitted frames
    enc28j60SetBank(ECON1);                                                 // switch to bank 0
    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE|EIE_PKTIE);      // enable interrutps
    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);             // enable packet reception
    /*
    cli();                                                                  //
    EICRA |= (1 << ISC21);                                                  // The falling edge of INT2 generates asynchronously an interrupt request.
    ENC28J60_INT_EN;
    //EIMSK |= (1 << INT2);                                                   // External Interrupt Request 2 Enable
    ENC28J60_INT_Flag = ENC28J60_INT_clr;                                   // Interrupt Flag lschen
    sei();                                                                  //
    */
}
// ***********************************************************************************************************************

// Pakete ber Ethernet raussenden ***************************************************************************************
void enc28j60PacketSend(unsigned int len, unsigned char* packet)            //
{                                                                           //
    unsigned char tsv_byte[7];                                              // Array fr Transmit Status Vector
    unsigned char rtx_count = 0;                                            // retransmit counter
    unsigned int count = 0;                                                 //
    for (count = 0; count < 50; count++)                                    //
    {                                                                       //
        if( !(enc28j60Read(ECON1) & ECON1_TXRTS) )                          //
        {                                                                   //
            break;                                                          //
        }                                                                   //
        _delay_ms(1);                                                       //
    }                                                                       //
    count = 0;                                                              //
    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRST);            // Reset transmit logic if a TX Error has previously occured
    enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRST);            // This is a silicon errata workaround
    enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, EIR, EIR_TXERIF | EIR_TXIF);    //
    enc28j60Write(EWRPTL, TXSTART_INIT&0xFF);                               // Set the write pointer to start of transmit buffer area
    enc28j60Write(EWRPTH, TXSTART_INIT>>8);                                 //
    enc28j60Write(ETXNDL, (TXSTART_INIT+len)&0xFF);                         // Set the TXND pointer to correspond to the packet size given
    enc28j60Write(ETXNDH, (TXSTART_INIT+len)>>8);                           //
    enc28j60WriteOp(ENC28J60_WRITE_BUF_MEM, 0, 0x00);                       // write per-packet control byte
    enc28j60WriteBuffer(len, packet);                                       // copy the packet into the transmit buffer
    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS);            // send the contents of the transmit buffer onto the network
    if (enc28j60Read(EREVID) == 0x05 || enc28j60Read(EREVID) ==0x06)        //
    {                                                                       //
        while(!(enc28j60Read(EIR) & (EIR_TXERIF | EIR_TXIF)) && (++count < 1000));//
        if ( (enc28j60Read(EIR) & EIR_TXERIF) || (count >= 1000) )          //
        {                                                                   //
            #ifdef _Debug_ENC                                               //
            print_debug_clock();                                            //
            printf_P(PSTR("ENC->TXERIF ausgeloest\r\n"));                   //
            #endif                                                          //
            enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRTS);    // Cancel the previous transmission if it has become stuck set
            enc28j60Write(ERDPTL, (TXSTART_INIT+len+1)&0xFF);               // Setzt den Lese Pointer auf den anfang des Transmit Status Vector
            enc28j60Write(ERDPTH, (TXSTART_INIT+len+1)>>8);                 //
            enc28j60ReadBuffer(7, tsv_byte);                                // Liest den Transmit Status Vector
            #ifdef _Debug_ENC                                               //
            print_debug_clock();                                            //
            printf_P(PSTR("ENC->TX Status Vector: %02X%02X:%02X:%02X:%02X%02X:%02X\r\n"),//
                tsv_byte[0],tsv_byte[1],tsv_byte[2],tsv_byte[3],            //
                tsv_byte[4],tsv_byte[5],tsv_byte[6]);                       //
            #endif                                                          //
            for(rtx_count = 0; rtx_count < 16; rtx_count++)                 //
            {                                                               //
                if(enc28j60Read(EIR) & EIR_TXERIF && tsv_byte[3] & tx_latecol)//
                {                                                           //
                    #ifdef _Debug_ENC                                       //
                    print_debug_clock();                                    //
                    printf_P(PSTR("ENC->TXERIF und Late Collision\r\n"));   //
                    #endif                                                  //
                    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRST);// Reset transmit logic if a TX Error has previously occured
                    enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRST);// This is a silicon errata workaround
                    enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, EIR, EIR_TXERIF | EIR_TXIF);//
                    #ifdef _Debug_ENC                                       //
                    print_debug_clock();                                    //
                    printf_P(PSTR("ENC->Reset TX Logic\r\n"));              //
                    #endif                                                  //
                    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS);// send the contents of the transmit buffer onto the network
                    while(!(enc28j60Read(EIR) & (EIR_TXERIF | EIR_TXIF)));  //
                    enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, ECON1_TXRTS);// Cancel the previous transmission if it has become stuck set
                    enc28j60Write(ERDPTL, (TXSTART_INIT+len+1)&0xFF);       // Setzt den Lese Pointer auf den anfang des Transmit Status Vector
                    enc28j60Write(ERDPTH, (TXSTART_INIT+len+1)>>8);         //
                    enc28j60ReadBuffer(7, tsv_byte);                        // Liest den Transmit Status Vector
                    #ifdef _Debug_ENC                                       //
                    print_debug_clock();                                    //
                    printf_P(PSTR("ENC->TX Status Vector: %02X%02X:%02X:%02X:%02X%02X:%02X"),//
                    tsv_byte[0],tsv_byte[1],tsv_byte[2],tsv_byte[3],        //
                    tsv_byte[4],tsv_byte[5],tsv_byte[6]);                   //
                    printf_P(PSTR(" --> Retry counter %d\r\n"),rtx_count);  //
                    #endif                                                  //
                }                                                           //
                else                                                        //
                {                                                           //
                    break;                                                  //
                }                                                           //
            }                                                               //
        }                                                                   //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************

// Pakete empfangen und in den Buffer schreiben **************************************************************************
unsigned int enc28j60PacketReceive(unsigned int maxlen, unsigned char* packet)
{                                                                           //
    unsigned int rsv/*, rs,re*/;                                            //
    unsigned int len;                                                       //
    if( !(enc28j60Read(EIR) & EIR_PKTIF) )                                  // prft ob Receive Int Flag gesetzt ist 
    if (enc28j60Read(EPKTCNT) == 0)                                         // pft ob der Paket counter = 0 ist
    return 0;                                                               // wenn Int Flag nicht gesetzt oder Paket counter 0 ist dann lnge 0 zurckgeben
    enc28j60Write(ERDPTL, (NextPacketPtr));                                 // Setzt den read pointer auf die Adresse die im NextPacketPtr steht
    enc28j60Write(ERDPTH, (NextPacketPtr)>>8);                              // Beim ersten durchlauf ist NextPacketPtr = RXSTART_INIT 0x0000
    NextPacketPtr   = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);             // Das erste Word im ReadBuffer ist die Adresse des nchsten Paketes
    NextPacketPtr  |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;          // 
    len             = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);             // Das zweite Word im Buffer ist die Framelnge des aktuellen Paketes
    len            |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;          // setzt sich zusammen aus: dest. adr, source adr., type/lenght,data,padding,crc
    rsv             = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);             // Das dritte Word im Buffer ist der receive status vector
    rsv            |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;          // 

    if( NextPacketPtr > RXSTOP_INIT ||                                      // Validate the data returned from the ENC28J60.  Random data corruption,
        (rsv & rsv_Zero) ||                                                 // such as if a single SPI bit error occurs while communicating or a 
        (rsv & rsv_CRCError) ||                                             // momentary power glitch could cause this to occur in rare circumstances.
        (len > maxlen) ||                                                   //
        (!(rsv & rsv_ReceiveOk)))                                           //
    {                                                                       //
        //#ifdef _Debug_ENC                                                 //
            printf_P(PSTR("\033[2J"));                                      // lscht den terminalbildschirm
            print_debug_clock();                                            //
            printf_P(PSTR("ENC->Receive error -> EIR:%02X -> Init ENC\r\n"),enc28j60Read(EIR));//
        //#endif                                                              //
        NextPacketPtr = 0;
        enc28j60Init();
        return 0;
    }                                                                       //
    /*
    if ( len > maxlen )                                                     // Wenn die Frmelnge grsser als maxlen ist dann :
    {                                                                       //
        enc28j60Write(ERXRDPTL, (NextPacketPtr));                           // Setzt den read pointer auf die Adr im NextPaketPtr
        enc28j60Write(ERXRDPTH, (NextPacketPtr)>>8);                        //
        enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);       // Decrementiert Packetcounter und zeigt das das Paket bearbeitet wurde
        return(0);                                                          // Paket wird verworfen weil die Framelnge grsser als der uC buffer ist
    }                                                                       //
    */
    enc28j60ReadBuffer(len, packet);                                        // Kopiert das Paket aus dem Receive Buffer in den uC buffer
                                                                            // an implementation of Errata B1 Section #13
    /*
    rs = enc28j60Read(ERXSTH);                                              // rs bekommt den wert aus ERXST RX Buffer Start
    rs <<= 8;                                                               //
    rs |= enc28j60Read(ERXSTL);                                             //
    re = enc28j60Read(ERXNDH);                                              // re bekommt den wert aus ERXND RX Buffer End
    re <<= 8;                                                               //
    re |= enc28j60Read(ERXNDL);                                             //
    if (NextPacketPtr - 1 < rs || NextPacketPtr - 1 > re)                   // Wenn Adr aus NextPacketPtr -1 kleiner ist als RX Buffer Start oder 
    {                                                                       // NextPacketPtr -1 grsser ist als RX Buffer End dann :
        enc28j60Write(ERXRDPTL, (re));                                      // RX Read Ptr = RX Buffer End
         enc28j60Write(ERXRDPTH, (re)>>8);                                  //
    }                                                                       //
    else                                                                    // Wenn nicht dann
    {                                                                       //
        enc28j60Write(ERXRDPTL, (NextPacketPtr-1));                         // RX Read Ptr = Adr aus NextPacketPtr -1
        enc28j60Write(ERXRDPTH, (NextPacketPtr-1)>>8);                      //
    }                                                                       //
    */
    
    //mark packet as processed (free mem)

/*
    enc28j60Write(EWRPTL, TXSTART_INIT&0xFF);                               // Set the write pointer to start of transmit buffer area
    enc28j60Write(EWRPTH, TXSTART_INIT>>8);                                 //
*/

    //ERRATA says we need to check packet pointer:
    if ((NextPacketPtr - 1 < RXSTART_INIT) || (NextPacketPtr - 1 > RXSTOP_INIT))
    {
        enc28j60Write(ERXRDPTL, (RXSTOP_INIT&0xFF));                         // RX Read Ptr = Adr aus NextPacketPtr -1
        enc28j60Write(ERXRDPTH, (RXSTOP_INIT)>>8);                      //
        //enc28j60_write_address(ENC28J60_REG_ERXRDPTL, LO8(ENC28J60_RX_BUFFER_END));
        //enc28j60_write_address(ENC28J60_REG_ERXRDPTH, HI8(ENC28J60_RX_BUFFER_END));
    }
    else
    {
        enc28j60Write(ERXRDPTL, (NextPacketPtr-1));                         // RX Read Ptr = Adr aus NextPacketPtr -1
        enc28j60Write(ERXRDPTH, (NextPacketPtr-1)>>8);                      //
        //enc28j60_write_address(ENC28J60_REG_ERXRDPTL, LO8(enc28j60_next_packet_ptr- 1));
        //enc28j60_write_address(ENC28J60_REG_ERXRDPTH, HI8(enc28j60_next_packet_ptr- 1));
    }

    enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);           // decrement the packet counter indicate we are done with this packet
    return (len-4);                                                         //
}                                                                           //
// ***********************************************************************************************************************

// Netzwerk Link prfen **************************************************************************************************
unsigned char link_check(void)                                              //
{                                                                           //
    if(enc28j60PhyRead(0x01) & PHSTAT1_LLSTAT)                              // Prfe ob Netzwerk Link vorhanden
    {                                                                       //
        return ( LINK_OK );                                                 //
    }                                                                       //
    else                                                                    // Wenn nicht dann
    {                                                                       //
        return ( LINK_DOWN );                                               //
    }                                                                       //
}                                                                           //
// ***********************************************************************************************************************
/*
// FUNKTION **************************************************************************************************************
ISR( INT2_vect )                                                            // RFM12 Interrupt an Int 0
{                                                                           // 
    unsigned char tmp_sreg;                                                 // temporaerer Speicher fuer das Statusregister
    tmp_sreg = SREG;                                                        // Statusregister (also auch das I-Flag darin) sichern
    cli();                                                                  // Interrupts global deaktivieren

    ENC28J60_INT_DIS;
    //EIMSK |= (1 << INT2);                                                   // External Interrupt Request 2 Enable
    ENC28J60_INT_Flag = ENC28J60_INT_set;                                   // Interrupt Flag setzen
    SREG = tmp_sreg;                                                        // Status-Register wieder herstellen somit auch das I-Flag auf gesicherten Zustand setzen
    sei();                                                                  // Interrupts ein
}                                                                           // 
// ***********************************************************************************************************************
*/